@@ -6,7 +6,7 @@ use POSIX;
 use List::Util;
 use Data::Dumper qw(Dumper);
 use Exporter qw(import);
-our $VERSION = '0.42';
+our $VERSION = '0.45';
 our @EXPORT_OK = qw(
     incr        reduces   flatten
     drop_right  drop      take_right  take
@@ -23,6 +23,14 @@ our @EXPORT_OK = qw(
 );
 use constant ARG_PLACE_HOLDER => {};
 
+$SIG{__WARN__} = sub {
+    caller eq "Sub::Fp" ? carp $_[0] : warn $_[0];
+};
+
+$SIG{__DIE__} = sub {
+    caller eq "Sub::Fp" ? croak $_[0] : die $_[0];
+};
+
 _wrap_to_use_partials(
     grep { $_ !~ /flow|flow_right|partial|chain/ } @EXPORT_OK
 );
@@ -43,8 +51,10 @@ sub _contains_placeholders {
 }
 
 sub _wrap {
+    ##no critic
     no warnings 'redefine';
     no strict 'refs';
+
     my ($methodName, $decorator) = @_;
     my $module                    = caller;
     my $fullName                  = "$module"."::"."$methodName";
@@ -70,7 +80,10 @@ sub _wrap_to_use_partials {
     }
 }
 
-sub noop { return undef }
+sub noop {
+    ## no critic;
+    return undef;
+}
 
 sub identity {
     my $args = shift // undef;
@@ -170,7 +183,7 @@ sub _safe_get {
 sub get {
     my ($path, $coll, $default) = @_;
 
-    if (!len($path)) {
+    if (!defined $path) {
         return $default;
     }
 
@@ -310,6 +323,7 @@ sub for_each {
         $fn->($val, $idx - 1, $coll);
     }
 
+    ##no critic;
     return undef;
 }
 
@@ -507,7 +521,7 @@ sub maps {
 
     my $idx = 0;
 
-    if (!is_sub($func) && $coll) {
+    if (!is_sub($func) && defined $coll) {
         return maps(get($func, __), $coll);
     }
 
@@ -671,6 +685,63 @@ concise code.
 
 =head1 SUBROUTINES/METHODS
 
+=head1 Self Partialing: The __ function
+
+Sub::Fp has a "special" function, the C<__> placeholder.
+This function allows for "self partialing", which is similar to auto-currying in many functional languages.
+This works for every single function in the Sub::Fp library, except for C<flow flow_right partial chain>
+
+    # TLDR
+
+    my $person = { name => "Sally" };
+
+    my $get_name = get('{name}', __);  # <---- Function uses placeholder to "self-partial", and return new sub
+
+    print $get_name->($person);        # <---- Invoke function with argument
+
+    # "Sally"
+
+That's it! It also works with multiple placeholders, in different positions,
+with different permutations of supplied arguments.
+
+    my $range = range(__, __);
+
+    print Dumper($range->(1, 10));
+
+    # [1,2,3,4,5,6,7,8,9]
+
+
+    my $range = range(__, __, __);
+
+    print Dumper($range->(1, 4, 0));
+
+    # [1,1,1,1];
+
+
+
+    my $range = range(__, 4, __);
+
+    print Dumper($range->(1, 0));
+
+    # [1,1,1,1];
+
+B<Prior Art>: This is not a new concept by any means (I'm just stealing it),
+and it's existed in functional languages for well over half a century.
+Its a natural application of functional composition.
+To get a better feel for what it looks like in other languages see:
+
+    #1 thread-as macro in Clojure
+
+    #2 Partialing in Lodash, PyToolz etc
+
+    #3 Auto Currying in Haskell, Lodash-Fp, Ramda, Elm
+
+4. The use of C<_> in languages as a placeholder. This library uses
+dobule underscore instead to differentiate it from the native library,
+which already uses a single underscore in some circumstances.
+
+=cut
+
 =head1 EXPORT
 
     incr         reduces   flatten
@@ -692,7 +763,7 @@ Increments the supplied number by 1
 
     incr(1)
 
-    # => 2
+    # 2
 
 =cut
 
@@ -702,7 +773,7 @@ Decrements the supplied number by 1
 
     decr(2)
 
-    # => 1
+    # 1
 
 =cut
 
@@ -738,7 +809,7 @@ arguments into the function it invokes
     apply($sum_all_nums, [100, 200]);
     # same as $sum_all_nums->(100, 200)
 
-    # => 300
+    # 300
 
 =cut
 
